
subroutine BC(N, T) 
!----------------------------------------------------------------------------------
! Set the boundary conditions for a discretized plate T with a 
! (N+2) x (N+2) mesh of nodes. The extremes at left/right/top/bottom of the plate
! are indexed by 0 and N+1.
!
! The boundaries are set to some nontrivial values, but are otherwise somewhat
! arbitrary. Changing them won't cause a major amount of difference in the 
! computational rates, which is what this batch of code is intended for.
!
! Corner points have some discontinuities, so the order in which the boundaries
! are set has effect on the final solution. However, it does not affect the 
! interior nodes since the corners never are used in the computation.
!
!----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!----------------
! Started: Thu 02 Feb 2017, 06:45 PM
! Last Modified: Sat 27 Jan 2018, 05:42 PM
!----------------------------------------------------------------------------------


    use iso_fortran_env, only: real8 => real64, stderr => error_unit
    implicit none

    integer :: N
    real(kind=real8) :: T(0:N+1,0:N+1)
    integer :: i, j
    real(kind=real8), parameter :: pi = &
        3.1415926535897932384626433832795028841971694_real8
    real(kind=real8) :: halfN , factor, x, y

    halfN = real((N+1),kind=real8)/2.0_real8
    factor = 2.0_real8*pi/halfN  ! = (4pi/(N+1))

    do i = 0,N+1
        ! Bottom of plate: two full sine curves on [-2pi, 2pi].
        ! The formula to convert from i-coordinates to x coordinates is 
        !      x = (i - (N+1)/2)*(4pi/(N+1))
        x = (real(i, kind=real8) - halfN)*factor
        T(i,0) = sin(x)

        ! Right-hand side of the plate: set to constant 1/2
        T(N+1,i) = 0.5_real8

        ! Top of the plate: put cosine curves. Why not?
        T(i,N+1)  = cos(x)

        ! Left-hand side of the plate: linear from 0 to 1
        y = real(i, kind=real8) / real(N+1,kind=real8)
        T(0,i) = y
    end do

end subroutine BC
