program jgs
!------------------------------------------------------------------------------------
! Solve heat transfer problem in a discretized plate, with mesh lines in both x and
! y directions numbered from 0 to N+1. The two extremes (0 and N+1) correspond to
! boundary conditions, so those are initialized but not updated during the
! diffusion process. The problem size and control parameters are specified in the
! file "parameters". This code can also output the final values of the plate
! temperatures to a file named "plate". The Matlab script plotplate.m will then
! read it in and, surprise, plot it. However, for performance analysis avoid calling
! printplate by setting the Boolean in "parameters" to .false.
!
! This has commented-out stubs for calling a Gauss-Seidel solver and OMP versions
! of both Jacobi and Gauss-Seidel. You need to fill those in, and then uncomment
! out calls to them in the switch/case construct.
!
!-----------------------
! Input/output files: 
!-----------------------
!
!  parameters: contains 6 control variables as input. Those are listed below.
!
!  results: a compressed output file with a single line per run. The file "header"
!       in this same directory give the column definitions. This file can be read
!       in by showresults.m and plotted. 
!
!  history: has iteration number and error measure for chosen iterates; see
!      the variable printfrequency. Can be plotted using the Matlab script 
!      plothistory.m.
!
!  plate: if showplate is true, will contain the final iterate. Can be plotted
!      using the Matlab script showplate.m
!
!--------------------
! Input parameters:
!--------------------
!  N = number of unknowns in both x and y direction. The plate size is actually
!      N+2 x N+2, when boundary nodes are also counted
!
!  maxits = maximum allowed number of iterations
!
!  tolerance = stopping test tolerance on the error measure, which is the 2-norm 
!      of the update size divided by N^2.
!
!  printfrequency = how often to write out the error measure. Set to a negative 
!      number to avoid having it printed out at all.
!
!  showplate = Boolean, whether or not to output the final iterate to a file
!
!  method = integer in range 1-4, specifying which method to run and time.
!               1 = Jacobi
!               2 = Gauss-Seidel
!               3 = Jacobi with OMP directives
!               4 = Gauss-Seidel with OMP directives
!
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!----------------------
! Last Modified: Thu 25 Jan 2018, 12:46 PM
!------------------------------------------------------------------------------------

    use iso_fortran_env, only: stdout => output_unit, stderr => error_unit, &
                               real8 => real64
    implicit none
    logical, parameter:: writesummary = .true. ! write summary info to std out

    integer :: i, j
    ! Names of the different methods that can be used
    character(len=20) :: methods(4) 
    ! The solution and update arrays
    real(kind=real8), target, allocatable ::  T(:,:), dT(:,:)
    ! Iteration control parameters
    logical :: showplate
    integer            :: N, maxits, results, method
    real(kind=real8)   :: tolerance, finalerror
    integer :: errflag, printfrequency, iterations
    ! Variables for timing information
    real(kind=real8) :: start, finish, timetaken
    real(kind=real8) , external :: elapsedtime
    real(kind=real8) :: gflops
    ! OMP stuff
    integer :: nthreads, k
    integer,external :: omp_get_num_threads, omp_get_thread_limit

    !===============================================================
    !-----------Start of executable statements----------------------
    !===============================================================
    methods(1) = 'Jacobi'
    methods(2) = 'Gauss-Seidel'
    methods(3) = 'OMP Jacobi'
    methods(4) = 'OMP Gauss-Seidel'

    call getparameters(N, maxits, tolerance, printfrequency, showplate, method)

    allocate( T(0:N+1,0:N+1), dT(0:N+1,0:N+1), stat=errflag )
    if (errflag /= 0) then
        write(unit=stderr,fmt=777) 'Unable to allocate needed memory; N = ', N, '.'
        stop
    end if

    ! Can only find the number of active threads in a parallel region
    !$OMP parallel
    nthreads = omp_get_num_threads()
    !$OMP end parallel

    if (writesummary) then
        write(stdout,777) 'Mesh parameter N = ', N
        write(stdout,777) 'Number of interior nodes = ', N*N
        write(stdout,777) 'Max allowed number of iterations = ', maxits
        if (printfrequency > 0) then
            write(stdout,777) 'printfrequency = ', printfrequency
        else
            write(stdout,666) 'Convergence history will not be saved'
        end if
        write(stdout,888) 'Stopping test tolerance = ', tolerance
        write(stdout,777) 'Number of omp threads avilable = ', nthreads
        write(stdout,777) 'Iterative method = ', method, ': '//methods(method) 
    end if
   
    call BC(N, T)           ! Set boundary conditions on border of T
    T(1:N,1:N) = 0.0_real8  ! Initialize interior nodes to all zeros
    dT = 0.0_real8          ! Zero out the diff array; only need the boundaries however

    start = elapsedtime()
    select case (method)
    case(1)
        call jacobi(N,T,dT,maxits,printfrequency,tolerance,iterations,finalerror)
!    case(2)
!        call gauss_seidel(N,T,dT,maxits,printfrequency,tolerance,iterations,finalerror)
!    case(3)
!        call omp_jacobi(N,T,dT,maxits,printfrequency,tolerance,iterations,finalerror)
!    case(4)
!        call omp_gauss_seidel(N,T,dT,maxits,printfrequency,tolerance,iterations,finalerror)
    case default
        write(stderr,777) 'Bad method number: ', method
        stop
    end select
    finish = elapsedtime()

    timetaken = finish - start
    gflops = real(iterations,kind=real8)*8.0e-9_real8*(real(N,kind=real8)**2)/timetaken

    ! Write results to a brief output file
    open(newunit=results,file='results',action='write',position='append',iostat=errflag)
    if (errflag /= 0) then
        write(stderr,777) 'Could not open results file; iostat = ', errflag
        stop
    end if
    write(results,999) N, nthreads, iterations, finalerror, timetaken, gflops, method
    close(results)

    if (writesummary) then
        write(stdout,777) 'Actual number of iterations = ', iterations
        write(stdout,888) 'Final error norm = ', finalerror
        write(stdout,888) 'Time taken = ', timetaken, ' secs'
        write(stdout,888) 'Computation rate = ', gflops, ' Gflops/sec'
        write(stdout,666) repeat('=', 84)
    end if


    if (showplate) call printplate(N,T)

    stop

    300 format(a,16x,a) 
    666 format(a)
    777 format(a, i0, a)
    888 format(a, es12.4, a)
    999 format(i0, 4x, i2, 5x, i0, 3x, es16.8, 3x, es16.8, 3x, es16.8, 3x, i1)

end program jgs
